// jhonyaku.c
//
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "macro.h"

#define iskanji(c) ((c) >= 0x81 && (c) <= 0x9F || (c) >= 0xE0 && (c) <= 0xFC)
#define MaxSent  2000

FILE *fpin, *fpout, *prcs, *dbg;
extern FILE *jed;
//char Dbg = 'Y';
char Dbg = 'N', BOTH = 'N', LOG = 'N';
char path[70], path_in[70], path_base[70];
char jps[JSentLen+MaxSent], jwd[JSentLen], wdw[JSentLen], wdh[JSentLen];
char aj[WordNumb][YakugoMax][JWordLen], az[WordNumb][YakugoMax][ZokuLen];
char caj[YakugoMax][JWordLen], caz[YakugoMax][ZokuLen];
char sj[80], oj[80], vr[100], sz[60], oz[60], vz[60];
char wdx[JWordLen], egs[SentLen+MaxSent], zkc[10];
char pst[JSentLen], en[WordNumb][JSentLen];
extern char cj[][JSentLen];
int  cnb, wnb, wc, ct, no;
int arrow = 0, html_f = 0, script = 0, style = 0;

main(argc, argv)
int  argc;
char *argv[];
{
    int ppt, i;
    char filnam[50], obj[50], *pt;

    strcpy(obj, "OBJECT");
    if(argc == 1){
      printf("Input File Name: ");
      scanf("%s", filnam);
    }
    else strcpy(filnam, argv[1]);

    if(argc >= 3){
      if(strcmp(argv[2], "-both") == 0) BOTH = 'Y';
      if(strcmp(argv[2], "-log") == 0) LOG = 'Y';
      else
        strcpy(obj, argv[2]);
    }

    if(argc >= 4){
      if(strcmp(argv[3], "-both") == 0) BOTH = 'Y';
      if(strcmp(argv[3], "-log") == 0) LOG = 'Y';
    }

    if(argc == 5){
      if(strcmp(argv[4], "-both") == 0) BOTH = 'Y';
      if(strcmp(argv[4], "-log") == 0) LOG = 'Y';
    }

    if(LOG == 'Y') Dbg = 'Y';
    else Dbg = 'N';

    fpin = fopen(filnam, "r");
    if(fpin == NULL) error("Input file not open!");
    fpout = fopen(obj, "w");
    jed = fopen("JEDIC.J2E", "r");
    if(jed == NULL) error("Dictionary file not open!");
    if(LOG == 'Y') prcs = fopen("PROCESS.J2E", "w");
    if(Dbg == 'Y') dbg = fopen("DEBUG.J2E", "w");
//    if(LOG == 'Y') fprintf(prcs, "LOG = %c Dbg = %c\n", LOG, Dbg);
    arrow = 0;
    while(1){
      if(GetSentence() != 0) break;

#ifdef __linux
      sjis2utf8(jps); // UTF-8ŏo
#else
      puts(jps);
#endif

      if(html_f != 0 || script > 0 || jps[strlen(jps)-1] == '>'
      || strncmp(jps, "/*", 2) == 0 
      || ((pt = (char*)strchr(jps, '@')) && isascii(*(pt-1)) && isascii(*(pt+1)))
      || strstr(jps, "&nbsp;") || strstr(jps, "&gt;") || strstr(jps, "&#") ){
        strcpy(egs, jps);
        if(jps[0] == '<') srcUrl();
      }
      else{
        if(style == 0) Jap2Eng();
        else strcpy(egs, jps);
        if(BOTH == 'Y') fprintf(fpout, "%s ", jps);  // N_̕\̏ꍇ
      }

      puts(egs);
      fprintf(fpout, "%s", egs);
      if(*egs != '<' || strstr(egs, "-->")) fprintf(fpout, "\n", egs);
    }
    if(Dbg == 'Y') fclose(dbg);
    if(LOG == 'Y') fclose(prcs);
    fclose(jed);
    fclose(fpout);
    fclose(fpin); 
  }

// htmlłsrc=ϊ
srcUrl(){
    char *search = "_external.url.path : ";
    char *search_in = "_internal.url.path : ";
    char *pt, *pt1, *pt2, dir[40];
    int  ln;
    strcpy(egs, jps);

    if(pt = (char*)strstr(jps, search)){
      ln = strlen(search);
      wdcpy(path, pt + ln);
      strcpy(path_base, path);
//      if(pt1 = (char*)strstr(path_base, "//")){
//        if(pt1 = (char*)strchr(pt1+2, '/')) *(pt1+1) = '\0';
//      }
      return;
    }
    if(pt = (char*)strstr(jps, search_in)){
      ln = strlen(search_in);
      wdcpy(path_in, pt + ln);
      return;
    }
    if(strncmp(jps, "<!--", 4) == 0 && script == 0) return;
//    if(strncmp(jps, "<?", 2) == 0) return;

    if(strstr(jps, "</STYLE>") || strstr(jps, "</style>")){
      style = 0;   return;
    }
    if(style != 0) return;

    if(strncmp(jps, "<STYLE", 6) == 0 || strncmp(jps, "<style", 6) == 0){
      style = 1;   return;
    }

        if(Dbg == 'Y') fprintf(dbg, "-- srcURL -- path = %s jps = %s \n", path_base, jps);
    if((pt = (char*)strstr(egs, "src=")) || (pt = (char*)strstr(egs, "SRC="))
    || (pt = (char*)strstr(egs, "BACKGROUND=")) || (pt = (char*)strstr(egs, "swfpath")) ){
      if((pt1 = (char*)strstr(pt, "\"/")) && pt1 < pt + 13){
        ptreplace(pt1+1, 1, path_base);
//        return;
      } else
      if((pt1 = (char*)strstr(pt, "\'/")) && pt1 < pt + 13){
        ptreplace(pt1+1, 1, path_base);
//        return;
      } else
      if((pt1 = (char*)strstr(pt, "\"./")) && pt1 < pt + 13){
        ptreplace(pt1+1, 2, path_base);
//        return;
      } else
      if(((pt1 = (char*)strstr(pt, "\"")) || (pt1 = (char*)strchr(pt, '\''))) && pt1 < pt + 13){
        if(strncmp(pt1+1, "http:", 5)){
          ptinsert(pt1+1, path_base);
//          return;
        }
      }
    }

//    if(script != 0) return;

    if((pt = (char*)strstr(egs, "<base ")) || (pt = (char*)strstr(egs, "<BASE "))){
//      if((pt1 = (char*)strchr(pt, '>'))){
//        ptinsert(pt1+1, "-->");
//        ptinsert(pt, "<!--");
//        return;
//      }
      if((pt1 = (char*)strstr(pt, "href=")) || (pt1 = (char*)strstr(pt, "HREF="))){
        return;
      }
    }
    if(((pt = (char*)strstr(egs, "<meta ")) || (pt = (char*)strstr(egs, "<META "))) && (strstr(egs, "Refresh") || strstr(egs, "refresh"))){
      if((pt1 = (char*)strchr(pt, '>'))){
        ptinsert(pt1+1, "-->");
        ptinsert(pt, "<!--");
        return;
      }
    }
    if((pt = (char*)strstr(egs, "<link ")) || (pt = (char*)strstr(egs, "<LINK "))){
      if((pt = (char*)strstr(egs, "href=")) || (pt = (char*)strstr(egs, "HREF="))){
        if((pt1 = (char*)strstr(pt, "\"/")) && pt1 < pt + 7){
          ptreplace(pt1+1, 1, "");
          ptinsert(pt1+1, path);
          if(Dbg == 'Y') fprintf(dbg, "-- srcURL2 -- path = %s egs = %s \n", path, egs);
          return;
        }
        if((pt1 = (char*)strchr(pt, '\"')) && pt1 < pt + 7){
          if(strncmp(pt1+1, "http:", 5) ){
          ptinsert(pt1+1, path);
          }
          if(Dbg == 'Y') fprintf(dbg, "-- srcURL2 -- path = %s egs = %s \n", path, egs);
          return;
        }
      }
      return;
    }

    if((pt = (char*)strstr(egs, "href=")) || (pt = (char*)strstr(egs, "HREF="))){
      if(((pt1 = (char*)strstr(pt, "\"/")) || (pt1 = (char*)strstr(pt, "\'/"))) && pt1 < pt + 7){
        ptreplace(pt1+1, 1, path_base);
        ptinsert(pt1+1, path_in);
        return;
      } else
      if((pt1 = (char*)strstr(pt, "\"./")) && pt1 < pt + 7){
        ptreplace(pt1+1, 2, path_base);
        ptinsert(pt1+1, path_in);
        return;
      } else
      if(((pt1 = (char*)strstr(pt, "\"")) || (pt1 = (char*)strchr(pt, '\''))) && pt1 < pt + 7){
        if(strncmp(pt1+1, "http:", 5) == 0){
          ptinsert(pt1+1, path_in);
          return;
        } else {
          ptinsert(pt1+1, path_base);
          ptinsert(pt1+1, path_in);
          return;
        }
      }
    }

    if((pt = (char*)strstr(egs, "charset=")) || (pt = (char*)strstr(egs, "CHARSET="))){
      if(pt1 = (char*)strstr(pt, "UTF-8")){
        ptreplace(pt1, 5, "ISO-20220-JP");
      }
    }

    if((pt = (char*)strstr(egs, "encoding=")) || (pt = (char*)strstr(egs, "ENCODING="))){
      if(pt1 = (char*)strstr(pt, "UTF-8")){
        ptreplace(pt1, 5, "ISO-20220-JP");
      }
    }

}

// |{̃Tu
Jap2Eng(){
    int  ppt, i;
    ajClear(0);
    if(strlen(jps) > JSentLen){
      if(Dbg == 'Y') fprintf(dbg, "-- Jap2Eng -- %s \n", jps);
      jps[JSentLen] = '\0';
    }
    strcpy(wdh, jps);
    no = 0;
    predit();

    for(wnb = 0 ; ; wnb++ ){
      JMP1:
      if(wdh[0] == '\0') break;
      strcpy(wdw, wdh);
      cnb = 0;
      strncpy(jwd, wdw, 2);
      if(isalnum(*jwd) || (ispunct(*jwd) && !isdigit(*(jwd+1)))
      || *jwd == '[' && isdigit(*(jwd+1)) || isspace(*jwd)) jwd[1] = '\0';
      jwd[2] = '\0';
      symbck();
      if(ct == 1){
        strcpy(wdx, jwd);    goto JMP;
      }
      wdx[0] = '\0';
      for(i = 0; ;i++ ){
        if(i >= 20) error("word cann't be searched !");
	if(wdx[0] != '\0') break;
        zkc[0] = '\0';
	preConj();
        if(strcmp(wdx, "(exit)") == 0) goto JMP1;
//        strncpy(jwd, wdw, 2);
//        jwd[2] = '\0';
        ppt = ptcal(wdw);
      if(Dbg == 'Y') fprintf(dbg, "-- main -- zkc = %s wdw = %s \n", zkc, wdw);
        diclook(ppt);
        if(wdx[0] == '\0') jishoFail();
      }
      JMP:
      right(wdw, wdh, strlen(wdw)-strlen(jwd));
    if(Dbg == 'Y') fprintf(dbg, "-- main -- wdh = %s \n", wdh);
      // "" or "" check
      if(strcmp(caj[0], "S") == 0 || strcmp(caj[0], "O") == 0) goto JMP1;
      strcpy(en[wnb], wdx);
  if(Dbg == 'Y') fprintf(dbg, "---main --- wdx = %s \n", wdx);
      for(i = 0; i <= cnb; i++){
        strcpy(aj[wnb][i], caj[i]);
        strcpy(az[wnb][i], caz[i]);
      }
      if(wdh[0] == '\0') break;
    }
    qstMark();
    yakDbg();
    tekyk();
    bunpo();
    siage();
}

int GetSentence()
{
    int  ch = 0, i = 0;
      int c;

    for(i = 0; i < JSentLen+MaxSent; i++) jps[i] = '\0';
    i = 0;
    if(arrow == 1){ jps[i++] = '<'; arrow = 0; }
    while(1){
      ch = getc(fpin);
      if(ch == EOF) break;
//      if(ch == ' ') continue;
      if(ch == 9) continue;
      if(html_f == 0 && script == 0 && (ch == '\n' || ch == '\r' || ch == 13)) continue;
      jps[i++] = ch;
      if(strstr(jps+i-2, "B") != 0 && jps[0] != '<') break;
      if(strstr(jps+i-2, "H") != 0 && jps[0] != '<') break;
      if(strstr(jps+i-2, "") != 0 && jps[0] != '<') break;
      // ](8b)@](93)@](91)
      // 0x5d
      if(strchr(jps+i-1, ']') && html_f != 0 // && (unsigned char)jps[i-2] <= 0x80
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                              && strncmp(jps+i-2, "]", 2)
                                                           ) break;
      if(strstr(jps, "<SCRIPT") || strstr(jps, "<script")) script = 1; 
      if((strstr(jps, "</SCRIPT>") || strstr(jps, "</script>")) && script == 1){ script = 0; break; }
      if(jps[i-2] == '<' && ch == '!') html_f++;
//      if(jps[i-2] == '-' && ch == '>' && html_f > 0){ html_f = 0; break; }
      if(ch == '>' && html_f > 0){ html_f = 0; break; }
      if(ch == '>' && html_f == 0) break;
      if(ch == '<' && html_f == 0){
        if(i > 1){ jps[i-1] = '\0'; arrow = 1; break; }
      }
      if(i >= JSentLen+MaxSent) break;
    }
    jps[i] = '\0';
//    fgets(jps, JSentLen, fpin);
  if(Dbg == 'Y') fprintf(dbg, "--- GetSentence --- jps = %s html_f = %d \n", jps, html_f);
    if(i == 0 && ch == EOF) return 1;
    return 0;
}

error(str)
char *str;
{
    printf("\n %s\n", str);
    exit(1);
}

jishoFail()
{
    if(Conj() == 1) return;
      ct = 0;
      numbck();
      if(ct == 1){
        strcpy(wdx, jwd);    return;
      }
    japcopy();
    strcpy(wdx, jwd);
}

ajClear(t)
int t;
{
    int i, j;
    for(i = t; i <= WordNumb-1; i++){
      for(j = 0; j <= YakMaxCt; j++){
        aj[i][j][0] = '\0';
        az[i][j][0] = '\0';
      }
//      en[i][0] = '\0';
    }
}

qstMark()
{
    int  i, j, cn;
    for(i = wnb; i >= 1; i--)
      if(aj[i][0][0] == '?'){
        for(j = i; j >= 0; j--){
          for(cn = 0; cn <= YakMaxCt; cn++){
            if(aj[j][cn][0] == '\0') break;
            else insertc(az[j][cn], 0, '?', strlen(az[j][cn]));
          }
        }
      }
}

yakDbg()
{
    int  i,j;

    if(LOG != 'Y') return;
    for(i = 0; i <= wnb; i++){
// printf("\nen[%d] = %s \n", i, en[i]);
      for(j = 0; j <= YakMaxCt; j++){
        if(aj[i][j][0] == '\0') break;
        fprintf(prcs, "aj[%d][%d] = %s    ", i, j, aj[i][j]);
        fprintf(prcs, "az[%d][%d] = %s  \n", i, j, az[i][j]);
      }
    }
}

yakAkitume(){
    int  i, j, ct;
    for(i = 0; i <= cnb; i++){
      if(caj[i][0] == '\0') break;
    }
    if(i > cnb) return;
    ct = i - 1;
    if(i == cnb){ cnb--;    return; }
    for(j = i+1; j <= cnb; j++)
      if(caj[j][0] != '\0'){
        ct++;
        strcpy(caj[ct], caj[j]);
        strcpy(caz[ct], caz[j]);
      }
    cnb = ct;
    caj[cnb+1][0] = '\0';
}


siage()
{
    int  ln, i;
    char *pt;
    ln = 0;
    egs[0] = '\0';
    for(i = 0; i <= wnb; i++){
      ln += strlen(cj[i]);
      if(ln > SentLen){
        printf("|󕶂Ȃ肷܂II\n");
        break;
      }
//  printf("--- siage --- cj[%d] = %s \n", i, cj[i]);
      ptcut(cj[i], 1);
      strcat(egs, cj[i]);    strcat(egs, " ");
    }
    pt = (char*)1;
    for(; pt != 0;){
      pt = (char*)strstr(egs, "  ");
      if(pt) ptreplace(pt, 2, " ");
    }
    pt = (char*)strstr(egs, " ,");
    if(pt) ptreplace(pt, 2, ",");
    pt = (char*)strstr(egs, " .");
    if(pt) ptreplace(pt, 2, ". ");
    pt = (char*)strstr(egs, " ?");
    if(pt) ptreplace(pt, 2, "? ");
    pt = (char*)strstr(egs, "is not");
    if(pt) ptreplace(pt, 6, "isn't");
    pt = (char*)strstr(egs, "was not");
    if(pt) ptreplace(pt, 7, "wasn't");
    if((pt = (char*)strstr(egs, "do not"))) ptreplace(pt, 6, "don't");
    if((pt = (char*)strstr(egs, "does not"))) ptreplace(pt, 8, "doesn't");
    if((pt = (char*)strstr(egs, "can not"))) ptreplace(pt, 7, "can't");
    if(isalnum(egs[0])) egs[0] = toupper(egs[0]);
    if(Dbg == 'Y') fprintf(dbg, "--- siage --- egs = %s \n", egs);
}

// ̃vGfBbg
predit()
{
    char commt[JSentLen], cmt2[JSentLen], dmy[5], *pt, *pt1;
    // JbȐ
    pt = wdh;
    if((pt = (char*)strstr(pt, "i")) && (pt1 = (char*)strstr(pt+2, "j"))){
      ptreplace(pt1, 2, ")");
      ptreplace(pt, 2, "(");
    }
//  fprintf(dbg, "--- predit --- wdh = %s \n", wdh);

    pt = wdh;
    if((pt = (char*)strstr(pt, "i")) && (pt1 = (char*)strchr(pt+2, ')'))){
      ptreplace(pt, 2, "(");
    }

    pt = wdh;
    if((pt = (char*)strchr(pt, '(')) && (pt1 = (char*)strstr(pt+2, "j"))){
      ptreplace(pt1, 2, ")");
    }

/*    for(pt = wdh; pt > 0; ){
      if(pt = (char*)strchr(pt, '(')){
        if(pt == wdh) break;
        if(strchr(pt+1, '(')) break;
        if((pt1 = (char*)strchr(pt + 1, ')')) || (pt1 = (char*)strchr(pt+1, '('))){
          if(*pt1 == ')'){
            ptcpy(commt, pt+1, pt1);
            strcpy(cmt2, pt1 + 1);
            dmy[0] = '*';
            dmy[1] = ++no + '0';
            dmy[2] = '\0';
            sprintf(pt, "%s%s%s%s", dmy, cmt2, dmy, commt);
            *pt = '%';
            pt = pt1;    continue;
          }
        }
        if(pt1 = (char*)strrchr(pt + 1, ')')){
          ptcpy(commt, pt+1, pt1);
          strcpy(cmt2, pt1 + 1);
          dmy[0] = '*';
          dmy[1] = ++no + '0';
          dmy[2] = '\0';
          sprintf(pt, "%s%s%s%s", dmy, cmt2, dmy, commt);
          *pt = '%';
        }
      }
    }*/

//    if(pt = (char*)strstr(wdh, "")) ptreplace(pt, 4, ""); // Ă邩炾
    if(pt = (char*)strstr(wdh, "")) ptreplace(pt, 4, " ");
    if(pt = (char*)strstr(wdh, "^")) ptreplace(pt, 2, "/");
    if(pt = (char*)strstr(wdh, "ƍKł")) ptreplace(pt, 10, "ĉ");
    if(pt = (char*)strstr(wdh, "āB")) ptreplace(pt, 6, "ĉB");
    if(pt = (char*)strstr(wdh, "ȂA")) ptreplace(pt+6, 4, "A");
    if((pt = (char*)strstr(wdh, "")) && strncmp(pt+6, "", 4)) ptreplace(pt, 6, "");
    if((pt = (char*)strstr(wdh, "Ȃ̂")) && strncmp(pt-4, "", 4)) ptreplace(pt, 2, "ł");
    if(pt = (char*)strstr(wdh, "ɏ")) ptreplace(pt, 8, "̏");
    if(pt = (char*)strstr(wdh, "̂")) ptreplace(pt, 8, "̌`");
    if(pt = (char*)strstr(wdh, "NȏO")) ptreplace(pt, 8, "NOȏ");
    if(pt = (char*)strstr(wdh, "ɂ͂܂")) ptreplace(pt, 4, "ɁA");
    if(pt = (char*)strstr(wdh, "")) ptreplace(pt, 4, "A");
    if(pt = (char*)strstr(wdh, "D")) ptreplace(pt, 6, "AD");
    if(pt = (char*)strstr(wdh, "܂")) ptreplace(pt, 8, "A܂");
    if(pt = (char*)strstr(wdh, "̂ł")) ptreplace(pt, 4, "/");
    if(pt = (char*)strstr(wdh, "̂Ƃ킴")) ptreplace(pt, 4, "/");
    if(pt = (char*)strstr(wdh, "Ƃ")) ptreplace(pt, 8, "");
    if(pt = (char*)strstr(wdh, "Ƃ͂Ă")) ptreplace(pt, 14, "A");
    if(strstr(wdh, "ǂꂭ炢") && (pt = (char*)strstr(wdh, "B")))
      ptreplace(pt, 2, "H");
    if(strstr(wdh, "Ȃ") && (pt = (char*)strstr(wdh, "B")))
      ptreplace(pt, 2, "H");
  if(Dbg == 'Y') fprintf(dbg, "--- predit --- ܂ -> Ȃ \n");
    if(pt = (char*)strstr(wdh, "܂")){
      if(strncmp(pt, "܂", 10)) ptreplace(pt, 8, "Ȃ");
    }
    if((pt = (char*)strstr(wdh, "B")) || (pt = (char*)strstr(wdh, "")) ){
      if(strncmp(pt-4, "", 4) && strncmp(pt-4, "", 4) && strncmp(pt-6, "Ƃ", 6) && strncmp(pt-4, "Ȃ", 4) )
        ptinsert(pt, "ł");
    }
//    if(pt = (char*)strstr(wdh, "v")) ptcut(pt, 2);
    for(pt = wdh; pt > 0; ){
      if(pt = (char*)strstr(wdh, "@")) ptcut(pt, 2);
    }
    for(pt = wdh; pt > 0; ){
      if(pt = (char*)strchr(wdh, ' ')) ptcut(pt, 1);
    }
/*    for(pt = wdh; pt > 0; ){
      if(pt = (char*)strchr(wdh, ' ')){
         if(isascii(*(pt-1)) && isalnum(*(pt+1))) ptcut(pt, 1);
         else pt++;
      }
    }*/
//  printf("--- predit --- wdh = %s \n", wdh);
}

